@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.assuredworkloads_v1beta1.services.assured_workloads_service import (
@@ -113,6 +118,42 @@ class AssuredWorkloadsServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return AssuredWorkloadsServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> AssuredWorkloadsServiceTransport:
         """Returns the transport used by the client instance.
@@ -176,18 +217,46 @@ class AssuredWorkloadsServiceAsyncClient:
 
     async def create_workload(
         self,
-        request: assuredworkloads_v1beta1.CreateWorkloadRequest = None,
+        request: Union[assuredworkloads_v1beta1.CreateWorkloadRequest, dict] = None,
         *,
         parent: str = None,
         workload: assuredworkloads_v1beta1.Workload = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
         r"""Creates Assured Workload.
 
+        .. code-block:: python
+
+            from google.cloud import assuredworkloads_v1beta1
+
+            def sample_create_workload():
+                # Create a client
+                client = assuredworkloads_v1beta1.AssuredWorkloadsServiceClient()
+
+                # Initialize request argument(s)
+                workload = assuredworkloads_v1beta1.Workload()
+                workload.display_name = "display_name_value"
+                workload.compliance_regime = "CA_REGIONS_AND_SUPPORT"
+
+                request = assuredworkloads_v1beta1.CreateWorkloadRequest(
+                    parent="parent_value",
+                    workload=workload,
+                )
+
+                # Make the request
+                operation = client.create_workload(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.assuredworkloads_v1beta1.types.CreateWorkloadRequest`):
+            request (Union[google.cloud.assuredworkloads_v1beta1.types.CreateWorkloadRequest, dict]):
                 The request object. Request for creating a workload.
             parent (:class:`str`):
                 Required. The resource name of the new Workload's
@@ -217,7 +286,7 @@ class AssuredWorkloadsServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, workload])
         if request is not None and has_flattened_params:
@@ -265,11 +334,11 @@ class AssuredWorkloadsServiceAsyncClient:
 
     async def update_workload(
         self,
-        request: assuredworkloads_v1beta1.UpdateWorkloadRequest = None,
+        request: Union[assuredworkloads_v1beta1.UpdateWorkloadRequest, dict] = None,
         *,
         workload: assuredworkloads_v1beta1.Workload = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> assuredworkloads_v1beta1.Workload:
@@ -278,8 +347,32 @@ class AssuredWorkloadsServiceAsyncClient:
         etag field in the Workload. Only one update operation per
         workload can be in progress.
 
+
+        .. code-block:: python
+
+            from google.cloud import assuredworkloads_v1beta1
+
+            def sample_update_workload():
+                # Create a client
+                client = assuredworkloads_v1beta1.AssuredWorkloadsServiceClient()
+
+                # Initialize request argument(s)
+                workload = assuredworkloads_v1beta1.Workload()
+                workload.display_name = "display_name_value"
+                workload.compliance_regime = "CA_REGIONS_AND_SUPPORT"
+
+                request = assuredworkloads_v1beta1.UpdateWorkloadRequest(
+                    workload=workload,
+                )
+
+                # Make the request
+                response = client.update_workload(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.assuredworkloads_v1beta1.types.UpdateWorkloadRequest`):
+            request (Union[google.cloud.assuredworkloads_v1beta1.types.UpdateWorkloadRequest, dict]):
                 The request object. Request for Updating a workload.
             workload (:class:`google.cloud.assuredworkloads_v1beta1.types.Workload`):
                 Required. The workload to update. The workload’s
@@ -311,7 +404,7 @@ class AssuredWorkloadsServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([workload, update_mask])
         if request is not None and has_flattened_params:
@@ -353,10 +446,10 @@ class AssuredWorkloadsServiceAsyncClient:
 
     async def delete_workload(
         self,
-        request: assuredworkloads_v1beta1.DeleteWorkloadRequest = None,
+        request: Union[assuredworkloads_v1beta1.DeleteWorkloadRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
@@ -364,8 +457,25 @@ class AssuredWorkloadsServiceAsyncClient:
         are already in a deleted state, otherwise the request will fail
         with a FAILED_PRECONDITION error.
 
+
+        .. code-block:: python
+
+            from google.cloud import assuredworkloads_v1beta1
+
+            def sample_delete_workload():
+                # Create a client
+                client = assuredworkloads_v1beta1.AssuredWorkloadsServiceClient()
+
+                # Initialize request argument(s)
+                request = assuredworkloads_v1beta1.DeleteWorkloadRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_workload(request=request)
+
         Args:
-            request (:class:`google.cloud.assuredworkloads_v1beta1.types.DeleteWorkloadRequest`):
+            request (Union[google.cloud.assuredworkloads_v1beta1.types.DeleteWorkloadRequest, dict]):
                 The request object. Request for deleting a Workload.
             name (:class:`str`):
                 Required. The ``name`` field is used to identify the
@@ -382,7 +492,7 @@ class AssuredWorkloadsServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -428,17 +538,36 @@ class AssuredWorkloadsServiceAsyncClient:
 
     async def get_workload(
         self,
-        request: assuredworkloads_v1beta1.GetWorkloadRequest = None,
+        request: Union[assuredworkloads_v1beta1.GetWorkloadRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> assuredworkloads_v1beta1.Workload:
         r"""Gets Assured Workload associated with a CRM Node
 
+        .. code-block:: python
+
+            from google.cloud import assuredworkloads_v1beta1
+
+            def sample_get_workload():
+                # Create a client
+                client = assuredworkloads_v1beta1.AssuredWorkloadsServiceClient()
+
+                # Initialize request argument(s)
+                request = assuredworkloads_v1beta1.GetWorkloadRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_workload(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.assuredworkloads_v1beta1.types.GetWorkloadRequest`):
+            request (Union[google.cloud.assuredworkloads_v1beta1.types.GetWorkloadRequest, dict]):
                 The request object. Request for fetching a workload.
             name (:class:`str`):
                 Required. The resource name of the Workload to fetch.
@@ -465,7 +594,7 @@ class AssuredWorkloadsServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -512,17 +641,37 @@ class AssuredWorkloadsServiceAsyncClient:
 
     async def list_workloads(
         self,
-        request: assuredworkloads_v1beta1.ListWorkloadsRequest = None,
+        request: Union[assuredworkloads_v1beta1.ListWorkloadsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListWorkloadsAsyncPager:
         r"""Lists Assured Workloads under a CRM Node.
 
+        .. code-block:: python
+
+            from google.cloud import assuredworkloads_v1beta1
+
+            def sample_list_workloads():
+                # Create a client
+                client = assuredworkloads_v1beta1.AssuredWorkloadsServiceClient()
+
+                # Initialize request argument(s)
+                request = assuredworkloads_v1beta1.ListWorkloadsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_workloads(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.assuredworkloads_v1beta1.types.ListWorkloadsRequest`):
+            request (Union[google.cloud.assuredworkloads_v1beta1.types.ListWorkloadsRequest, dict]):
                 The request object. Request for fetching workloads in an
                 organization.
             parent (:class:`str`):
@@ -548,7 +697,7 @@ class AssuredWorkloadsServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -599,11 +748,17 @@ class AssuredWorkloadsServiceAsyncClient:
         # Done; return the response.
         return response
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
         gapic_version=pkg_resources.get_distribution(
-            "google-cloud-assuredworkloads",
+            "google-cloud-assured-workloads",
         ).version,
     )
 except pkg_resources.DistributionNotFound:
