@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,6 +37,7 @@ __protobuf__ = proto.module(
 
 class CreateWorkloadRequest(proto.Message):
     r"""Request for creating a workload.
+
     Attributes:
         parent (str):
             Required. The resource name of the new Workload's parent.
@@ -60,6 +61,7 @@ class CreateWorkloadRequest(proto.Message):
 
 class UpdateWorkloadRequest(proto.Message):
     r"""Request for Updating a workload.
+
     Attributes:
         workload (google.cloud.assuredworkloads_v1beta1.types.Workload):
             Required. The workload to update. The workload’s ``name``
@@ -78,6 +80,7 @@ class UpdateWorkloadRequest(proto.Message):
 
 class DeleteWorkloadRequest(proto.Message):
     r"""Request for deleting a Workload.
+
     Attributes:
         name (str):
             Required. The ``name`` field is used to identify the
@@ -95,6 +98,7 @@ class DeleteWorkloadRequest(proto.Message):
 
 class GetWorkloadRequest(proto.Message):
     r"""Request for fetching a workload.
+
     Attributes:
         name (str):
             Required. The resource name of the Workload to fetch. This
@@ -109,6 +113,7 @@ class GetWorkloadRequest(proto.Message):
 
 class ListWorkloadsRequest(proto.Message):
     r"""Request for fetching workloads in an organization.
+
     Attributes:
         parent (str):
             Required. Parent Resource to list workloads from. Must be of
@@ -134,6 +139,7 @@ class ListWorkloadsRequest(proto.Message):
 
 class ListWorkloadsResponse(proto.Message):
     r"""Response of ListWorkloads endpoint.
+
     Attributes:
         workloads (Sequence[google.cloud.assuredworkloads_v1beta1.types.Workload]):
             List of Workloads under a given parent.
@@ -154,6 +160,13 @@ class Workload(proto.Message):
     r"""An Workload object for managing highly regulated workloads of
     cloud customers.
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         name (str):
             Optional. The resource name of the workload.
@@ -181,27 +194,35 @@ class Workload(proto.Message):
             Output only. Immutable. The Workload creation
             timestamp.
         billing_account (str):
-            Required. Input only. The billing account used for the
-            resources which are direct children of workload. This
-            billing account is initially associated with the resources
-            created as part of Workload creation. After the initial
-            creation of these resources, the customer can change the
-            assigned billing account. The resource name has the form
+            Input only. The billing account used for the resources which
+            are direct children of workload. This billing account is
+            initially associated with the resources created as part of
+            Workload creation. After the initial creation of these
+            resources, the customer can change the assigned billing
+            account. The resource name has the form
             ``billingAccounts/{billing_account_id}``. For example,
             ``billingAccounts/012345-567890-ABCDEF``.
         il4_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.IL4Settings):
             Required. Input only. Immutable. Settings
             specific to resources needed for IL4.
+
+            This field is a member of `oneof`_ ``compliance_regime_settings``.
         cjis_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.CJISSettings):
             Required. Input only. Immutable. Settings
             specific to resources needed for CJIS.
+
+            This field is a member of `oneof`_ ``compliance_regime_settings``.
         fedramp_high_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.FedrampHighSettings):
             Required. Input only. Immutable. Settings
             specific to resources needed for FedRAMP High.
+
+            This field is a member of `oneof`_ ``compliance_regime_settings``.
         fedramp_moderate_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.FedrampModerateSettings):
             Required. Input only. Immutable. Settings
             specific to resources needed for FedRAMP
             Moderate.
+
+            This field is a member of `oneof`_ ``compliance_regime_settings``.
         etag (str):
             Optional. ETag of the workload, it is
             calculated on the basis of the Workload
@@ -211,11 +232,10 @@ class Workload(proto.Message):
             Optional. Labels applied to the workload.
         provisioned_resources_parent (str):
             Input only. The parent resource for the resources managed by
-            this Assured Workload. May be either an organization or a
-            folder. Must be the same or a child of the Workload parent.
-            If not specified all resources are created under the
-            Workload parent. Formats: folders/{folder_id}
-            organizations/{organization_id}
+            this Assured Workload. May be either empty or a folder
+            resource which is a child of the Workload parent. If not
+            specified all resources are created under the parent
+            organization. Format: folders/{folder_id}
         kms_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.KMSSettings):
             Input only. Settings used to create a CMEK
             crypto key. When set a project with a KMS CMEK
@@ -227,6 +247,19 @@ class Workload(proto.Message):
             properties (such as custom project id) will be
             used to create workload resources if possible.
             This field is optional.
+        kaj_enrollment_state (google.cloud.assuredworkloads_v1beta1.types.Workload.KajEnrollmentState):
+            Output only. Represents the KAJ enrollment
+            state of the given workload.
+        enable_sovereign_controls (bool):
+            Optional. Indicates the sovereignty status of
+            the given workload. Currently meant to be used
+            by Europe/Canada customers.
+        saa_enrollment_response (google.cloud.assuredworkloads_v1beta1.types.Workload.SaaEnrollmentResponse):
+            Output only. Represents the SAA enrollment
+            response of the given workload. SAA enrollment
+            response is queried during GetWorkload call. In
+            failure cases, user friendly error message is
+            shown in SAA details page.
     """
 
     class ComplianceRegime(proto.Enum):
@@ -239,9 +272,18 @@ class Workload(proto.Message):
         US_REGIONAL_ACCESS = 5
         HIPAA = 6
         HITRUST = 7
+        EU_REGIONS_AND_SUPPORT = 8
+        CA_REGIONS_AND_SUPPORT = 9
+
+    class KajEnrollmentState(proto.Enum):
+        r"""Key Access Justifications(KAJ) Enrollment State."""
+        KAJ_ENROLLMENT_STATE_UNSPECIFIED = 0
+        KAJ_ENROLLMENT_STATE_PENDING = 1
+        KAJ_ENROLLMENT_STATE_COMPLETE = 2
 
     class ResourceInfo(proto.Message):
         r"""Represent the resources that are children of this Workload.
+
         Attributes:
             resource_id (int):
                 Resource identifier. For a project this represents
@@ -254,7 +296,9 @@ class Workload(proto.Message):
             r"""The type of resource."""
             RESOURCE_TYPE_UNSPECIFIED = 0
             CONSUMER_PROJECT = 1
+            CONSUMER_FOLDER = 4
             ENCRYPTION_KEYS_PROJECT = 2
+            KEYRING = 3
 
         resource_id = proto.Field(proto.INT64, number=1,)
         resource_type = proto.Field(
@@ -263,6 +307,7 @@ class Workload(proto.Message):
 
     class KMSSettings(proto.Message):
         r"""Settings specific to the Key Management Service.
+
         Attributes:
             next_rotation_time (google.protobuf.timestamp_pb2.Timestamp):
                 Required. Input only. Immutable. The time at
@@ -285,6 +330,7 @@ class Workload(proto.Message):
 
     class IL4Settings(proto.Message):
         r"""Settings specific to resources needed for IL4.
+
         Attributes:
             kms_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.KMSSettings):
                 Required. Input only. Immutable. Settings
@@ -297,6 +343,7 @@ class Workload(proto.Message):
 
     class CJISSettings(proto.Message):
         r"""Settings specific to resources needed for CJIS.
+
         Attributes:
             kms_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.KMSSettings):
                 Required. Input only. Immutable. Settings
@@ -309,6 +356,7 @@ class Workload(proto.Message):
 
     class FedrampHighSettings(proto.Message):
         r"""Settings specific to resources needed for FedRAMP High.
+
         Attributes:
             kms_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.KMSSettings):
                 Required. Input only. Immutable. Settings
@@ -321,6 +369,7 @@ class Workload(proto.Message):
 
     class FedrampModerateSettings(proto.Message):
         r"""Settings specific to resources needed for FedRAMP Moderate.
+
         Attributes:
             kms_settings (google.cloud.assuredworkloads_v1beta1.types.Workload.KMSSettings):
                 Required. Input only. Immutable. Settings
@@ -344,12 +393,54 @@ class Workload(proto.Message):
                 Indicates the type of resource. This field should be
                 specified to correspond the id to the right project type
                 (CONSUMER_PROJECT or ENCRYPTION_KEYS_PROJECT)
+            display_name (str):
+                User-assigned resource display name.
+                If not empty it will be used to create a
+                resource with the specified name.
         """
 
         resource_id = proto.Field(proto.STRING, number=1,)
         resource_type = proto.Field(
             proto.ENUM, number=2, enum="Workload.ResourceInfo.ResourceType",
         )
+        display_name = proto.Field(proto.STRING, number=3,)
+
+    class SaaEnrollmentResponse(proto.Message):
+        r"""Signed Access Approvals (SAA) enrollment response.
+
+        Attributes:
+            setup_status (google.cloud.assuredworkloads_v1beta1.types.Workload.SaaEnrollmentResponse.SetupState):
+                Indicates SAA enrollment status of a given
+                workload.
+
+                This field is a member of `oneof`_ ``_setup_status``.
+            setup_errors (Sequence[google.cloud.assuredworkloads_v1beta1.types.Workload.SaaEnrollmentResponse.SetupError]):
+                Indicates SAA enrollment setup error if any.
+        """
+
+        class SetupState(proto.Enum):
+            r"""Setup state of SAA enrollment."""
+            SETUP_STATE_UNSPECIFIED = 0
+            STATUS_PENDING = 1
+            STATUS_COMPLETE = 2
+
+        class SetupError(proto.Enum):
+            r"""Setup error of SAA enrollment."""
+            SETUP_ERROR_UNSPECIFIED = 0
+            ERROR_INVALID_BASE_SETUP = 1
+            ERROR_MISSING_EXTERNAL_SIGNING_KEY = 2
+            ERROR_NOT_ALL_SERVICES_ENROLLED = 3
+            ERROR_SETUP_CHECK_FAILED = 4
+
+        setup_status = proto.Field(
+            proto.ENUM,
+            number=1,
+            optional=True,
+            enum="Workload.SaaEnrollmentResponse.SetupState",
+        )
+        setup_errors = proto.RepeatedField(
+            proto.ENUM, number=2, enum="Workload.SaaEnrollmentResponse.SetupError",
+        )
 
     name = proto.Field(proto.STRING, number=1,)
     display_name = proto.Field(proto.STRING, number=2,)
@@ -388,10 +479,16 @@ class Workload(proto.Message):
     resource_settings = proto.RepeatedField(
         proto.MESSAGE, number=15, message=ResourceSettings,
     )
+    kaj_enrollment_state = proto.Field(proto.ENUM, number=17, enum=KajEnrollmentState,)
+    enable_sovereign_controls = proto.Field(proto.BOOL, number=18,)
+    saa_enrollment_response = proto.Field(
+        proto.MESSAGE, number=20, message=SaaEnrollmentResponse,
+    )
 
 
 class CreateWorkloadOperationMetadata(proto.Message):
     r"""Operation metadata to give request details of CreateWorkload.
+
     Attributes:
         create_time (google.protobuf.timestamp_pb2.Timestamp):
             Optional. Time when the operation was
@@ -404,6 +501,10 @@ class CreateWorkloadOperationMetadata(proto.Message):
             Optional. Compliance controls that should be
             applied to the resources managed by the
             workload.
+        resource_settings (Sequence[google.cloud.assuredworkloads_v1beta1.types.Workload.ResourceSettings]):
+            Optional. Resource properties in the input
+            that are used for creating/customizing workload
+            resources.
     """
 
     create_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
@@ -412,6 +513,9 @@ class CreateWorkloadOperationMetadata(proto.Message):
     compliance_regime = proto.Field(
         proto.ENUM, number=4, enum="Workload.ComplianceRegime",
     )
+    resource_settings = proto.RepeatedField(
+        proto.MESSAGE, number=5, message="Workload.ResourceSettings",
+    )
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
